import { propertyTypes } from "../../models/property/PropertyType";
import { FormBuilder } from "../../components/FormBuilder";
import { DynamicArrayField } from "./components/DynamicArrayField";
import React from "react";
import { getValidationMarkForProperty } from "../..";
import { buildPropertyRenderInfo } from "../helpers/BuildPropertyRenderProps";
export const createComplexObjectBuilder = (labelRender, validationMessage, loadingSpinner) => {
    const id = "internal_complexbuilder";
    const DefaultLabelRender = (props) => React.createElement("label", { key: props.parentKey + "-label" }, props.propertySchema.displayName);
    const defaultValidationMessageElement = (message) => React.createElement("p", null, message);
    const defaultSpinner = (props, context) => React.createElement("p", null, "Loading...");
    const BuilderLabelRender = labelRender ?? DefaultLabelRender;
    const builderValidationMessageElement = validationMessage ?? defaultValidationMessageElement;
    const loadingSpinnerComponent = () => loadingSpinner ?? defaultSpinner;
    const convertPropertyOverrides = (propertyOverrides, property) => {
        if (propertyOverrides == undefined)
            return undefined;
        let clone = { ...propertyOverrides };
        if (propertyOverrides.disabledProps != null) {
            if (Array.isArray(propertyOverrides.disabledProps)) {
                if (propertyOverrides.disabledProps.indexOf(property) >= 0) {
                    clone.disabledProps = true;
                }
            }
            else {
                clone.disabledProps = !!propertyOverrides.disabledProps;
            }
        }
        console.log(clone);
        return clone;
    };
    const build = (renderProps, property, schema) => {
        let { item, validationResults } = renderProps;
        if (item == null)
            throw Error("item is null");
        if (schema == null)
            throw Error("schema is null");
        let info = buildPropertyRenderInfo(renderProps, schema, property);
        let validationMark = getValidationMarkForProperty(renderProps, property);
        const WrapInLabel = (element, addErrormessage) => {
            return (React.createElement("div", { className: "formbuilder-property", key: info.props.key },
                React.createElement(BuilderLabelRender, { key: `${info.key}-labelcontainer`, propertySchema: schema, hideLabel: info.props.options.hideLabel, parentKey: info.key, validationMark: validationMark }),
                element,
                (addErrormessage && info.props.errorMessage) && builderValidationMessageElement(info.props.errorMessage)));
        };
        const propertyType = propertyTypes;
        switch (schema.propertyType) {
            case propertyType.array:
                return { found: true, element: WrapInLabel(React.createElement(DynamicArrayField, { ...schema, ...info.props })) };
            case propertyType.custom:
                let customConfig = schema.config;
                if (!customConfig?.component)
                    return { found: false, element: undefined };
                // const Comp = this.customComponents[customConfig?.componentName];
                const Comp = customConfig.component;
                return { found: true, element: React.createElement(React.Fragment, null, WrapInLabel(React.createElement(Comp, { key: info.key, ...info.props, ...schema, ...customConfig.componentProps }))) };
            case propertyType.formItem:
                let formConfig = schema.config;
                if (!formConfig?.schemaConfig)
                    return { found: false, element: undefined };
                let formItemProps = {
                    item: item[property],
                    schemaConfig: formConfig.schemaConfig,
                    groupContainer: formConfig.groupContainer,
                    groupRender: formConfig.groupRender,
                    onPropertyChange: (item, prop, value) => info.props.onChange(item),
                    validationResult: validationResults,
                    validationResultPrefix: property,
                    propertyOverrides: convertPropertyOverrides(renderProps.propertyOverrides, property)
                };
                // handle dynamic schema coming from formitem config
                if (formConfig.schemaConfig.dynamicSchema != null) {
                    const dynamicSchema = formConfig.schemaConfig = formConfig.schemaConfig.dynamicSchema(item);
                    formConfig.schemaConfig = dynamicSchema;
                    formItemProps.schemaConfig = dynamicSchema;
                }
                let addErrormessage = false;
                // used for having properties as array of formitem
                let listProps = schema?.listProps;
                if (listProps != undefined) {
                    formItemProps.onPropertyChange = undefined;
                    listProps.config.onItemsChange = items => info.props.onChange(items);
                    addErrormessage = true;
                }
                // types sucks sometimes!! (ewi)
                return { found: true, element: React.createElement(React.Fragment, null, WrapInLabel(React.createElement(FormBuilder, { key: info.key, keyPrefix: info.key, ...formItemProps, ...formConfig, listProps: listProps }), addErrormessage)) };
            default: return { found: false, element: undefined };
        }
    };
    return { id, build, loadingComponent: loadingSpinnerComponent };
};
//# sourceMappingURL=ComplexObjectBuilder.js.map