import { useEffect, useState } from 'react';
import { FormBuilder } from '../../../components/FormBuilder';
import { ValidationEventType } from '../../../models/validation/ValidationEventType';
import { ValidationMark } from '../../../models/validation/ValidationMark';
import React from "react";
import { FormBuilderListEditorType } from '../../../interfaces/lists/IFormBuilderListEditorConfig';
import { IFormBuilderListMenuItemSelectionMode } from '../../../interfaces/lists/IFormBuilderListMenuConfig';
const wrapValues = (values) => values?.map(_ => { return { value: _ }; });
const unwrapValues = (fakeItems) => fakeItems?.map(_ => _.value);
export const DynamicArrayField = (props) => {
    const [items, setItems] = useState(wrapValues(props.value) ?? []);
    useEffect(() => setItems(wrapValues(props.value) ?? []), [props.value]);
    const getSchema = () => {
        var schema = {
            options: {
                validation: {
                    ignoreValidation: false,
                    validationRules: {
                        value: { id: "required", validationMark: ValidationMark.Required, validationRule: (item) => item.value != null && item.value != "", validationMessage: "Value must be specified", validateOn: ValidationEventType.Blur | ValidationEventType.Manual },
                    }
                },
                properties: {
                    value: { propertyType: props.config?.propertyType, displayName: "Value", config: { resizable: true } },
                }
            }
        };
        return schema;
    };
    const onChange = (fakeItems) => {
        const values = unwrapValues(fakeItems);
        props.onChange(values);
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(FormBuilder, { item: items, schemaConfig: { schemaProvider: { key: "fakeit", getSchema: async () => getSchema() } }, listProps: {
                config: {
                    itemIdentifier: (item) => item?.value?.toString(),
                    multiSelect: true,
                    onItemsChange: (fakeItems) => onChange(fakeItems),
                },
                editorConfig: {
                    title: item => "Item",
                    containerOptions: {
                        type: FormBuilderListEditorType.Dialog
                    }
                },
                menuConfig: {
                    actions: (newItem, editItem, deleteItems) => ([
                        { title: "New", iconName: "Add", selectionMode: IFormBuilderListMenuItemSelectionMode.None | IFormBuilderListMenuItemSelectionMode.Single | IFormBuilderListMenuItemSelectionMode.Multi, action: () => newItem() },
                        { title: "Edit", iconName: "Edit", selectionMode: IFormBuilderListMenuItemSelectionMode.Single, action: () => editItem() },
                        { title: "Delete", iconName: "Delete", selectionMode: IFormBuilderListMenuItemSelectionMode.Single | IFormBuilderListMenuItemSelectionMode.Multi, action: () => deleteItems() },
                    ]),
                }
            }, overrideSchema: getSchema() })));
};
//# sourceMappingURL=DynamicArrayField.js.map