import React, { useEffect, useImperativeHandle, useRef, useState } from 'react';
import { ValidationEventType } from '../models/validation/ValidationEventType';
import { Form } from './Form';
import { FormList } from './FormList';
import { formbuilder } from '../utils/FormBuilderInitializer';
import { schemaFromConfig } from '../utils/schema/schemaFromConfig';
// wrapper for handling both single item and list of items
// export const FormBuilder = forwardRef(<T extends IFormItem, FormBuilderRef>(props : IFormBuilderProps<T>, ref: FormBuilderRef) => {
export const FormBuilder = (props) => {
    // verify that the formbuilder is set up correctly
    formbuilder.verify();
    const [item, setItem] = useState(props.item);
    const subFormRef = useRef();
    const [schema, setSchema] = useState(props.overrideSchema);
    useImperativeHandle(props.formRef, () => ({
        getItem: () => subFormRef.current?.getItem(),
        validateItem: () => subFormRef.current?.validateItem != null ? subFormRef.current?.validateItem(ValidationEventType.Manual) : undefined
    }), []);
    const loadSchema = async () => {
        let schemaMerged = await schemaFromConfig(props.schemaConfig, props.formItemConfigOverrides, props.propertyOverrides);
        setSchema(schemaMerged);
    };
    useEffect(() => {
        setItem(props.item);
    }, [props.item]);
    useEffect(() => {
        if (props.overrideSchema == null)
            loadSchema();
    }, [props.schemaConfig]);
    if (schema == null)
        return null;
    const getKey = () => {
        return `${props.keyPrefix}`;
    };
    return (React.createElement("div", { className: "formbuilder-rootcontainer", key: `${props.keyPrefix}-formbuilder-container` }, (Array.isArray(props.item) || props.listProps != null)
        ? React.createElement(FormList, { key: getKey(), ref: subFormRef, items: item, schema: schema, ...props, listProps: props.listProps ?? schema.options.listOptions, keyPrefix: `${props.keyPrefix}-list`, readOnly: props.propertyOverrides?.disabledProps === true })
        : React.createElement(Form, { key: getKey(), ref: subFormRef, schema: schema, item: item, onPropertyChange: (item, prop, value) => props.singleItemProps?.onPropertyChange == null ? () => { } : props.singleItemProps.onPropertyChange(item, prop, value), ...props, keyPrefix: getKey() })));
};
//# sourceMappingURL=FormBuilder.js.map