export const buildFormItemRender = () => {
    var _builders = [];
    const hasBuilders = () => _builders.length > 0;
    const register = (builder) => {
        if (_builders.findIndex(_ => _.id === builder.id) >= 0)
            return;
        _builders.push(builder);
    };
    const registerRange = (builders) => {
        builders?.forEach(_ => register(_));
    };
    const list = () => {
        for (const builder of _builders) {
            if (builder.listComponent)
                return builder.listComponent();
        }
        return undefined;
    };
    const loadingComponent = () => {
        for (const builder of _builders) {
            if (builder.loadingComponent)
                return builder.loadingComponent();
        }
        return undefined;
    };
    const properties = (renderProps, properties) => {
        let elements = [];
        // loop through properties specified or defautls to all properties
        for (const prop of properties ?? Object.keys(renderProps.schema.properties)) {
            let propSchema = { ...renderProps.schema.properties[prop] };
            if (propSchema.hide && propSchema.hide(renderProps.item))
                continue;
            // check if property type is not available in any builders
            let found = false;
            for (const builder of _builders) {
                let result = builder.build(renderProps, prop, propSchema);
                if (result.found) {
                    found = true;
                    if (result.element != null)
                        elements.push(result.element);
                }
            }
            if (!found)
                console.warn(`Property type '${propSchema.propertyType}' is not supported in any registered builders. Maybe you've forgot to register a builder?`);
        }
        return elements?.length ? elements : undefined;
    };
    return { register, registerRange, properties, hasBuilders, list, loadingComponent };
};
//# sourceMappingURL=FormItemRender.js.map