import { formSchemaUtil } from "../../utils/common/FormSchemaUtil";
const validateRule = async (item, property, rule, onEvent, results, validationResultPrefix) => {
    if (rule == null)
        return;
    let propertyIndexer = validationResultPrefix ? `${validationResultPrefix}.${property}` : property;
    if (Array.isArray(rule)) {
        rule.forEach(_ => {
            let validated = (onEvent != undefined && !((_.validateOn & onEvent) === onEvent)) || (!_.disabled && _.validationRule?.(item));
            if (results[propertyIndexer] == null)
                results[propertyIndexer] = {};
            results[propertyIndexer][_.id] = { message: validated ? "" : _.validationMessage, success: !!validated };
        });
    }
    else {
        if (rule.nestedValidation != undefined) {
            let childSchema = rule.nestedValidation.schemaKey
                ? await formSchemaUtil.getSchemaFromMap(rule.nestedValidation.schemaKey)
                : await rule.nestedValidation.schemaProvider?.getSchema();
            if (item[property] != null) {
                results = await formValidator.validate(item[property], undefined, childSchema, onEvent, results, property); // TODO: maybe set all events here ('undefined'), to prevent validation messages to dissapear when other properties change (ewi)
            }
        }
        else {
            let validated = (onEvent != undefined && !((rule.validateOn & onEvent) === onEvent)) || (!rule.disabled && rule.validationRule?.(item));
            if (results[propertyIndexer] == null)
                results[propertyIndexer] = {};
            results[propertyIndexer][rule.id] = { message: validated ? "" : rule.validationMessage, success: !!validated };
        }
    }
};
export const formValidator = {
    // TODO: handle nested validation on complex objects, more smooth (ewi)
    // TODO: handle validation differently than using 'onEvent'?! (ewi)
    validate: async (item, prop, schema, onEvent, existingResults, validationResultPrefix) => {
        let validation = schema?.options?.validation;
        if (validation == null)
            return existingResults;
        if (validation.ignoreValidation)
            return existingResults;
        let rules = validation.validationRules;
        if (rules == null)
            return existingResults;
        let results = existingResults ?? {};
        // specific prop needs validation
        if (prop !== undefined) {
            let rule = rules[prop];
            await validateRule(item, prop, rule, onEvent, results, validationResultPrefix);
        }
        else {
            for (const key of Object.keys(rules)) {
                let rule = rules[key];
                await validateRule(item, key, rule, onEvent, results, validationResultPrefix);
            }
        }
        return results;
    }
};
//# sourceMappingURL=FormValidator.js.map