import { useCallback, useRef, useState } from "react";
const isFunction = (setStateAction) => typeof setStateAction === "function";
// type UseStateRef = {
//     <S>(initialState: S | (() => S)): [S, Dispatch<SetStateAction<S>>, ReadOnlyRefObject<S>];
//     <S = undefined>(): [S | undefined, Dispatch<SetStateAction<S | undefined>>, ReadOnlyRefObject<S | undefined>];
// };
/**
 * Hook for adding a ref to the regular usestate hook.
 * The ref is automatically updated when setstate is called.
 * @returns a regular usestate where a ref is included
 */
export const useStateRef = (initialState) => {
    const [state, setState] = useState(initialState); // sometimes types are fucked in TS (ewi) // TODO!!
    const ref = useRef(state);
    const dispatch = useCallback((setStateAction) => {
        ref.current = isFunction(setStateAction) ? setStateAction(ref.current) : setStateAction;
        setState(ref.current);
    }, []);
    return [state, dispatch, ref];
};
//# sourceMappingURL=useStateRef.js.map