import { buildFormItemRender } from "../components/helpers/FormItemRender";
import { initLanguage } from "../models/language/Language";
import { strings } from "./common/String";
import { createComplexObjectBuilder } from "../builders/custom/ComplexObjectBuilder";
const schemaRegister = {};
export const formbuilder = {
    formItemRender: undefined,
    initialize: () => {
        formbuilder.formItemRender = buildFormItemRender();
        return formbuilder;
    },
    usingComplexBuilder: (labelRender, validationMsgElement, loadingSpinner) => {
        formbuilder.formItemRender?.register(createComplexObjectBuilder(labelRender, validationMsgElement, loadingSpinner));
        return formbuilder;
    },
    withBuilders: (...builders) => {
        formbuilder.formItemRender?.registerRange(builders);
        return formbuilder;
    },
    withLanguage: (language) => {
        initLanguage(language);
        return formbuilder;
    },
    registerSchemaProvider: (provider) => {
        if (provider == null || strings.isNullOrWhiteSpace(provider.key))
            return;
        schemaRegister[provider.key] = provider;
    },
    getSchemaProvider: (key) => {
        return schemaRegister[key];
    },
    verify: () => {
        if (formbuilder.formItemRender == null) // TODO: add link to init guide
            throw Error("Formbuilder not initialized! Call 'formBuilder.initialize' as early as possible");
        if (!formbuilder.formItemRender.hasBuilders()) // TODO: add link to builders guide
            throw Error("Formbuilder hasn't got any builders! Please add one ore more builders. Either the standard ones or your own customs using 'withBuilders' and 'usingComplexBuilder'.");
        // all is well
    }
};
//# sourceMappingURL=FormBuilderInitializer.js.map