import { FormListColumnsPickType } from "../..";
import { formSchemaUtil } from "./FormSchemaUtil";
import { mergeDeep } from "./MergeObjects";
export const fetchSchemaFromMap = async (schemaMapKey, formItemConfigOverrides, propertyOverrides, dynamicKey) => {
    let schema = await formSchemaUtil.getSchemaFromMap(schemaMapKey, dynamicKey);
    return mergeSchema(schema, formItemConfigOverrides, propertyOverrides, dynamicKey);
};
export const fetchSchema = async (schemaProvider, formItemConfigOverrides, propertyOverrides, dynamicKey) => {
    let schema = await schemaProvider.getSchema(dynamicKey);
    return mergeSchema(schema, formItemConfigOverrides, propertyOverrides, dynamicKey);
};
export const mergeSchema = (schema, formItemConfigOverrides, propertyOverrides, dynamicKey) => {
    if (schema == null)
        return undefined;
    // merge with overrides
    let merged = mergeDeep(schema?.options, formItemConfigOverrides);
    // property overrides
    let disabledOverrides = propertyOverrides?.disabledProps;
    let hiddenOverrides = propertyOverrides?.hiddenProps;
    Object.keys(merged.properties).forEach(_ => {
        if (disabledOverrides != null) {
            let disable = merged.properties[_].disable;
            if (Array.isArray(disabledOverrides)) {
                if (disabledOverrides.indexOf(_) >= 0) {
                    disable = () => true;
                }
            }
            else {
                disable = () => !!disabledOverrides;
            }
            merged.properties[_].disable = disable;
        }
        if (hiddenOverrides != null) {
            const found = hiddenOverrides.columns.indexOf(_) >= 0;
            if (hiddenOverrides.pickType == FormListColumnsPickType.Only) {
                merged.properties[_].hide = () => found;
            }
            else if (hiddenOverrides.pickType == FormListColumnsPickType.Without) {
                merged.properties[_].hide = () => !found;
            }
        }
    });
    schema.options = merged;
    return schema;
};
//# sourceMappingURL=SchemaFetch.js.map